-------------localisation-------------
local L = AceLibrary("AceLocale-2.2"):new("GoodTarget")

L:RegisterTranslations("enUS", function()
	return {
		["GoodTarget"] = true,
	}
end)

-------------create Ace stuff-------------

GoodTarget = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

---------------saved variables------------

GoodTarget:RegisterDB("GoodTargetDB", "GoodTargetDBPC")
GoodTarget:RegisterDefaults("profile", {
    CheckInterval = 0.05,
    PlaySound = true,
    isLocked = false,
    Scale = 1.0,
    Width = 100,
    Height = 20,
    BGAlpha = 0.0,
    partyEnabled = true,
    partyClickThrough = false,
} )

-------------------Options---------------
local options = { 
    type='group',
    args = {
        
        perfHeader = {
            type = 'header',
            name = 'Performance',
            order = 1,
        },
        checkInterval = {
            type = 'range',
            name = 'Check interval',
            order = 2,
            desc = 'Set the interval in which GoodTarget checks the targets target (in seconds).',
            min = 0.01,
            max = 1.0,
            step = 0.01,
            bigStep = 0.02,            
            get = function() return GoodTarget.db.profile.CheckInterval end,
            set = "SetCheckInterval",
        },
        
        ---options---
        miscHeader = {
            type = 'header',
            name = 'Miscellaneous',
            order = 19,
        },
        playSound = {
            type = 'toggle',
            name = 'Play sounds',
            order = 20,
            desc = 'Enables or disables sounds on aggro loss or gain.',
            get = function() return GoodTarget.db.profile.PlaySound end,
            set = function(v) GoodTarget.db.profile.PlaySound = v end,
        },
        isLocked = {
            type = 'toggle',
            name = 'Lock frames',
            order = 21,
            desc = 'Locks or unlocks the frames.',
            get = function() return GoodTarget.db.profile.isLocked end,
            set = "SetLocked",
        },
        
        ---party frames---
        partyHeader = {
            type = 'header',
            name = 'Party frames',
            order = 39,
        },
        partyClickThrough = {
            type = 'toggle',
            name = 'Click-throughable-mode',
            order = 40,
            desc = 'Makes the frames click throughable, but you won\'t be able to click-to-target them anymore (Needs reload ui to update).',
            get = function() return GoodTarget.db.profile.partyClickThrough end,
            set = "SetPartyClickThrough",
        },
        partyEnabled = {
            type = 'toggle',
            name = 'Enable party frame',
            order = 40,
            desc = 'Enables the party targets frame.',
            get = function() return GoodTarget.db.profile.partyEnabled end,
            set = "SetPartyEnabled",
        },
        bgAlpha = {
            type = 'range',
            name = 'party background alpha',
            order = 41,
            desc = 'Sets the visibility of the background of the party targets.',
            min = 0.00,
            max = 1.0,
            step = 0.01,
            bigStep = 0.02,
            get = function() return GoodTarget.db.profile.BGAlpha end,
            set = "SetBGAlpha",
        },
        
        ---Size---
        appHeader = {
            type = 'header',
            name = 'Appearance',
            order = 59,
        },
        scale = {
            type = 'range',
            name = 'Scale',
            order = 60,
            desc = 'Set the scale of the frames.',
            min = 0.1,
            max = 4.0,
            step = 0.01,
            bigStep = 0.1,
            get = function() return GoodTarget.db.profile.Scale end,
            set = "SetScale",
        },
        width = {
            type = 'range',
            name = 'Width',
            order = 61,
            desc = 'Set the width of the frames.',
            min = 10,
            max = 500,
            step = 1,
            bigStep = 1,
            get = function() return GoodTarget.db.profile.Width end,
            set = "SetWidth",
        },
        height = {
            type = 'range',
            name = 'Height',
            order = 62,
            desc = 'Set the height of the frames.',
            min = 5,
            max = 100,
            step = 1,
            bigStep = 1,
            get = function() return GoodTarget.db.profile.Height end,
            set = "SetHeight",
        },
    },
}

GoodTarget:RegisterChatCommand("/GoodTarget", "/gt", options)
----------options getter and setter-----------

---locked---
function GoodTarget:SetLocked(isLocked)
    self.db.profile.isLocked = isLocked
    self:LockFrame(isLocked)
    if self.lastPlayerTTType == self.NO_TARGET then
        self:SetNonePlayerFrame()
    end
end

---party enabled---
function GoodTarget:SetPartyClickThrough(partyClickThrough)
    self.db.profile.partyClickThrough = partyClickThrough
end

---party enabled---
function GoodTarget:SetPartyEnabled(partyEnabled)
    self.db.profile.partyEnabled = partyEnabled
    self:LockFrame(partyEnabled)
    if partyEnabled then
        self.partyFrame:Show();
    else
        self.partyFrame:Hide()
    end
end

---scale---
function GoodTarget:SetScale(scale)
    self.db.profile.Scale = scale
    self.playerFrame:SetScale(scale)
    self:SetPartyFrameScale(scale)
end

---width---
function GoodTarget:SetWidth(width)
    self.db.profile.Width = width
    self.playerFrame:SetWidth(width)
    self:SetPartyFrameWidth(width)
end

---height---
function GoodTarget:SetHeight(height)
    self.db.profile.Height = height
    self.playerFrame:SetHeight(height)
    self:SetPartyFrameHeight(height)
end

---check interval---
function GoodTarget:SetCheckInterval(interval)
    self.db.profile.CheckInterval = interval
    self:CancelScheduledEvent("GTCheckTargets")
    self:ScheduleRepeatingEvent("GTCheckTargetsID", "GTCheckTargets", interval)
end

---background alpha---
function GoodTarget:SetBGAlpha(alpha)
    self.db.profile.BGAlpha = alpha
    self:LockFrame(self.db.profile.isLocked)
end

---------------initialisation--------------

function GoodTarget:OnInitialize()

    -----constants-----
    self.NO_TARGET = 0
    self.GOOD_TARGET = 1
    self.BAD_TARGET = 2
    
    -----init members----
    self.lastPlayerTTarget = "$none"
    self.lastPlayerTTType = self.NO_TARGET
    
    self.lastPartyTTarget = {}    
    self.lastPartyTTType = {}
    
    for id = 1, 4, 1 do
        self.lastPartyTTarget["party"..id] = "$none"
        self.lastPartyTTType["party"..id] = self.NO_TARGET
    end
    
    
    -----create frames-----
    self:CreatePlayerTargetFrame()
    self:CreatePartyTargetFrame()
    
    
    -----init frames-----
    self:InitializeFrames()
    
end

function GoodTarget:OnEnable()
    ---register events---
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:ScheduleRepeatingEvent("GTCheckTargetsID", "GTCheckTargets", self.db.profile.CheckInterval)    
    self:RegisterEvent("GTCheckTargets")
    
    
    ---show frames---
    --self:ShowFrames()
    
    ---update---
end

function GoodTarget:OnDisable()
    ---hide frames---
    self:HideFrames()
end

-----------------event handling-----------------
--[[Starts moving the frame on mouse down]]
function GoodTarget:OnMouseDown(f, button)
    if button == "LeftButton" then
		f:StartMoving()
	end
end

--[[Stops moving the frame on mouse up]]
function GoodTarget:OnMouseUp(f)
    f:StopMovingOrSizing()
end

--[[called whenever the current target of the player changes]]
function GoodTarget:PLAYER_TARGET_CHANGED()
    self:Update(true)
end

--[[This event is called in a specific interval and indicates
    to update the targets]]
function GoodTarget:GTCheckTargets()
    self:Update()
end


--------------actual functionality--------------

---general---
--[[Checks the targets and updates the frames]]
function GoodTarget:Update(playerTargetChanged)
    if self:PlayerTTargetChanged() then
        self:UpdatePlayerTTarget()
    end
    
    if self.db.profile.partyEnabled then
        self:UpdatePartyTTargets(playerTargetChanged)
    end
end

---party targets---

function GoodTarget:UpdatePartyTTargets(playerTargetChanged)
    local partyMembers = GetNumPartyMembers()
    if partyMembers == 0 then
        return
    end
    
    for i = 1, partyMembers, 1 do
        if playerTargetChanged or self:PartyTTargetChanged("party"..i) then
            self:UpdatePartyMembersTTarget("party"..i)
        end
    end
end

function GoodTarget:UpdatePartyMembersTTarget(unit)
    local tTargetType = self:ClassifyPartyTarget(unit)
    
    if tTargetType == self.GOOD_TARGET then
        self:SetGoodPartyFrame(unit)
    elseif  tTargetType == self.BAD_TARGET then
        if tTargetType ~= self.lastPartyTTType[unit] then --set bad frame layout if changed
            self:SetBadPartyFrame(unit)
            if self.db.profile.PlaySound then
                PlaySoundFile("Sound\\Creature\\Cannon\\CannonWoundB.wav")
            end
        end
        
        --set name and color
        local text = self.partyFrame.members[unit].Text
        text:SetText(self.lastPartyTTarget[unit])
        local _, class = UnitClass(unit.."targettarget")
        text:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b, 1)       
    else
        self:SetGoodPartyFrame(unit)
    end
    
    self.lastPartyTTType[unit] = tTargetType
end

--[[Classifies the targets target of a party member as
    good or bad.]]
function GoodTarget:ClassifyPartyTarget(unit)
    local unitTT = unit.."targettarget"
    local unitT = unit.."target"
    if not UnitExists(unitTT) then
        return self.GOOD_TARGET
    elseif UnitIsUnit("player", unitTT) or UnitIsUnit("playertarget", unitT) or not UnitIsEnemy("player", unitT) then
        return self.GOOD_TARGET
    elseif UnitInParty(unitTT) or UnitInRaid(unitTT) then
        for j = 1, 4, 1 do
            if unit == ("party"..j) then
                return self.BAD_TARGET
            end
            if UnitIsUnit(unitT, "party"..j.."target") then
                return self.GOOD_TARGET
            end
        end
        return self.BAD_TARGET
    else
        return self.GOOD_TARGET
    end
end


--[[Checks if the targets target of the party has changed.
    Returns true if the players targets target has changed.]]
function GoodTarget:PartyTTargetChanged(unit)
    local tTarget = UnitName(unit.."targettarget")
    if not tTarget then
        tTarget = "$none"
    end
        
    if tTarget == self.lastPartyTTarget[unit] then
        return false
    end
    
    self.lastPartyTTarget[unit] = tTarget
    return true
end



---player target---
--[[Checks the current target of the player and updates the frame]]
function GoodTarget:UpdatePlayerTTarget()
    local tTargetType = self:ClassifyPlayerTarget()
    
    if tTargetType == self.GOOD_TARGET then
        self:SetGoodPlayerFrame()
        if self.db.profile.PlaySound then
            PlaySoundFile("Sound\\interface\\AuctionWindowOpen.wav");
        end
    elseif  tTargetType == self.BAD_TARGET then
        if tTargetType ~= self.lastPlayerTTType then --set bad frame layout if changed
            self:SetBadPlayerFrame()
            if self.db.profile.PlaySound then
                PlaySoundFile("Sound\\Creature\\Cannon\\CannonDeath.wav")
            end
        end
        
        --set name and color
        local text = self.playerFrame.Text
        text:SetText(self.lastPlayerTTarget)
        local _, class = UnitClass("playertargettarget")
        text:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b, 1)       
    else
        self:SetNonePlayerFrame()
    end
    
    self.lastPlayerTTType = tTargetType
end


--[[Checks if the targets target of the player has changed.
    Returns true if the players targets target has changed.]]
function GoodTarget:PlayerTTargetChanged()
    local tTarget = UnitName("playertargettarget")
    if not tTarget then
        tTarget = "$none"
    end
        
    if tTarget == self.lastPlayerTTarget then
        return false
    end
    
    self.lastPlayerTTarget = tTarget
    return true
end

--[[Classifies the targets target of the player as
    Good, bad or none.]]
function GoodTarget:ClassifyPlayerTarget()
    if not UnitExists("playertargettarget") or not UnitIsEnemy("player", "playertarget") then
        return self.NO_TARGET
    elseif UnitIsUnit("player", "playertargettarget") then
        return self.GOOD_TARGET
    elseif UnitInParty("playertargettarget") or UnitInRaid("playertargettarget") then
        return self.BAD_TARGET
    else
        return self.NO_TARGET
    end
end


------------------GUI stuff-----------------

------frame creation------

---player frame---
function GoodTarget:CreatePlayerTargetFrame()
    local f=CreateFrame("FRAME","GoodTargetPlayerFrame",UIParent);
            
    f:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      tile=1, tileSize=10, edgeSize=0, 
      insets={left=0, right=0, top=0, bottom=0}
    });
    
    f:SetPoint("CENTER", UIParent)   
    f:SetMovable(true)
    
    f.master = self
    f:SetScript("OnMouseUp", function(f) f.master:OnMouseUp(f) end)
	f:SetScript("OnMouseDown", function(f, button) f.master:OnMouseDown(f, button) end)
    
    f.Text = f:CreateFontString("$parentText","ARTWORK","GameFontNormal");
    f.Text:SetPoint("CENTER",f)
    
    self.playerFrame = f
end

---party frame---
function GoodTarget:CreatePartyTargetFrame()
    local f=CreateFrame("FRAME","GoodTargetPartyFrame",UIParent);
            
    f:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      tile=1, tileSize=10, edgeSize=0, 
      insets={left=0, right=0, top=0, bottom=0}
    });
    
    f:SetPoint("CENTER", UIParent)  
    f:SetMovable(true)
    
    f.master = self
    f:SetScript("OnMouseUp", function(f) f.master:OnMouseUp(f) end)
	f:SetScript("OnMouseDown", function(f, button) f.master:OnMouseDown(f, button) end)
    
    f.Text = f:CreateFontString("$parentText","ARTWORK","GameFontNormal");
    f.Text:SetPoint("CENTER",f)
    
    f.members = {}
    
    for i = 1, 4, 1 do
        local id = "party"..i
        local pf
        if self.db.profile.partyClickThrough then
            pf = self:CreatePartyMemberFrameSimple(id, f)
        else
            pf = self:CreatePartyMemberFrame(id, f)
        end
        
        if i == 1 then
            pf:SetPoint("TOPLEFT", f)
        else
            pf:SetPoint("TOPLEFT", f.members["party"..(i - 1)], "BOTTOMLEFT")
        end
        
        f.members[id] = pf
    end
    
    self.partyFrame = f
end

function GoodTarget:CreatePartyMemberFrame(id, parent)
    local f = CreateFrame("BUTTON", "$Member"..id, parent, "SecureUnitButtonTemplate")
    
    f:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      tile=1, tileSize=10, edgeSize=0, 
      insets={left=0, right=0, top=0, bottom=0}
    });
    
    
    f.Text = f:CreateFontString("$parentText","ARTWORK","GameFontNormal");
    f.Text:SetPoint("CENTER",f)
   
    f:SetAttribute("type", "target")
    f:SetAttribute("unit", id.."target")
    
    f:Show()
    return f
end

function GoodTarget:CreatePartyMemberFrameSimple(id, parent)
    local f = CreateFrame("BUTTON", "$Member"..id, parent)
    
    f:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      tile=1, tileSize=10, edgeSize=0, 
      insets={left=0, right=0, top=0, bottom=0}
    });
    
    
    f.Text = f:CreateFontString("$parentText","ARTWORK","GameFontNormal");
    f.Text:SetPoint("CENTER",f)
    
    f:EnableMouse(false)
    f:Show()
    return f
end

-----frame visibility-----

function GoodTarget:ShowFrames()
    self.playerFrame:Show()
    self.partyFrame:Show()
end

function GoodTarget:HideFrames()
    self.playerFrame:Hide()
    self.partyFrame:Hide()
end


-----frame appeareance-----

---general---
function GoodTarget:InitializeFrames()
    local settings = self.db.profile
    
    if self.db.profile.partyEnabled then
        self.partyFrame:Show()
    else
        self.partyFrame:Hide()
    end
    
    self:LockFrame(settings.isLocked)
    
    local f = self.playerFrame
    f:SetWidth(settings.Width)
    f:SetHeight(settings.Height)
    f:SetScale(settings.Scale)
    self:SetNonePlayerFrame()
    
    local f = self.partyFrame
    self:SetPartyFrameWidth(settings.Width)
    self:SetPartyFrameHeight(settings.Height)
    self:SetPartyFrameScale(settings.Scale)
    
    for i = 1, 4, 1 do
        self:SetGoodPartyFrame("party"..i)
    end
    
    self.playerFrame:Show()
end

function GoodTarget:LockFrame(locked)
    local f = self.playerFrame    
    f:EnableMouse(not locked)
    
    local f = self.partyFrame    
    f:EnableMouse(not locked)
    
    if locked then
        f:SetBackdropColor(0, 0, 0, self.db.profile.BGAlpha)
        f.Text:SetTextColor(0, 0, 0, 0)
        for i = 1, 4, 1 do
            f.members["party"..i]:Show()
        end
    else
        f:SetBackdropColor(0.5, 0.5, 0.5, 0.5)
        f.Text:SetTextColor(1, 1, 1, 1)
        f.Text:SetText("lock me to hide me")
        for i = 1, 4, 1 do
            f.members["party"..i]:Hide()
        end
    end
end

---player frame---

function GoodTarget:SetGoodPlayerFrame()
    self.playerFrame:SetBackdropColor(0, 1, 0, 1)
    self.playerFrame.Text:SetTextColor(1, 1, 1, 1)
    self.playerFrame.Text:SetText("good")
end

function GoodTarget:SetBadPlayerFrame()
    self.playerFrame:SetBackdropColor(1, 0, 0, 1)
end

function GoodTarget:SetNonePlayerFrame()
    if self.db.profile.isLocked then
        self:SetLockedPlayerFrame()
    else        
        self:SetUnlockedPlayerFrame()
    end
end

function GoodTarget:SetLockedPlayerFrame()
    self.playerFrame:SetBackdropColor(0, 0, 0, 0)
    self.playerFrame.Text:SetTextColor(0, 0, 0, 0)
end

function GoodTarget:SetUnlockedPlayerFrame()
    self.playerFrame:SetBackdropColor(0.5, 0.5, 0.5, 0.5)
    self.playerFrame.Text:SetTextColor(1, 1, 1, 1)
    self.playerFrame.Text:SetText("lock me to hide me")
end

---party frame---

function GoodTarget:SetBadPartyFrame(unit)
    self.partyFrame.members[unit]:SetBackdropColor(1, 0, 0, 1)
end

function GoodTarget:SetGoodPartyFrame(unit)
    self.partyFrame.members[unit]:SetBackdropColor(1, 0, 0, 0)
    self.partyFrame.members[unit].Text:SetTextColor(1, 1, 1, 0)
end

function GoodTarget:SetPartyFrameScale(scale)
    self.partyFrame:SetScale(scale)
end

function GoodTarget:SetPartyFrameHeight(height)
    local f = self.partyFrame
    f:SetHeight(height * 5)
    
    for i = 1, 4, 1 do
        f.members["party"..i]:SetHeight(height)
    end
end

function GoodTarget:SetPartyFrameWidth(width)
    local f = self.partyFrame
    f:SetWidth(width)
    
    for i = 1, 4, 1 do
        f.members["party"..i]:SetWidth(width)
    end
end






